=== modified file 'loggerhead/__init__.py'
--- loggerhead/__init__.py	2008-12-22 19:54:58 +0000
+++ loggerhead/__init__.py	2009-02-10 21:11:43 +0000
@@ -12,6 +12,19 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""Just a simple container to turn this into a python package."""
+"""A simple container to turn this into a python package.
+
+We also check the versions of some dependencies.
+"""
+
+import pkg_resources
+
 __version__ = '1.10'
 required_bzrlib = (1, 6)
+
+pkg_resources.get_distribution('Paste>=1.6')
+try:
+    pkg_resources.get_distribution('PasteDeploy>=1.3')
+except pkg_resources.DistributionNotFound:
+    # No paste.deploy is OK, but an old paste.deploy is bad.
+    pass

=== modified file 'loggerhead/apps/branch.py'
--- loggerhead/apps/branch.py	2009-02-06 05:36:03 +0000
+++ loggerhead/apps/branch.py	2009-02-18 10:36:48 +0000
@@ -23,10 +23,13 @@
 from loggerhead import util
 
 
+_DEFAULT = object()
+
 class BranchWSGIApp(object):
 
     def __init__(self, branch, friendly_name=None, config={},
-                 graph_cache=None, branch_link=None, is_root=False):
+                 graph_cache=None, branch_link=None, is_root=False,
+                 served_url=_DEFAULT):
         self.branch = branch
         self._config = config
         self.friendly_name = friendly_name
@@ -36,6 +39,7 @@
             graph_cache = bzrlib.lru_cache.LRUCache()
         self.graph_cache = graph_cache
         self.is_root = is_root
+        self.served_url = served_url
 
     def get_history(self):
         _history = History(self.branch, self.graph_cache)
@@ -70,11 +74,6 @@
         kw = util.get_context(**kw)
         return self.url(*args, **kw)
 
-    @property
-    def served_url(self):
-        """The URL that you can fetch this branch from."""
-        return self.url([])
-
     def static_url(self, path):
         return self._static_url_base + path
 
@@ -103,6 +102,8 @@
         if self._static_url_base is None:
             self._static_url_base = self._url_base
         self._environ = environ
+        if self.served_url is _DEFAULT:
+            self.served_url = self.url([])
         path = request.path_info_pop(environ)
         if not path:
             raise httpexceptions.HTTPMovedPermanently(

=== modified file 'loggerhead/apps/filesystem.py'
--- loggerhead/apps/filesystem.py	2009-01-23 19:00:35 +0000
+++ loggerhead/apps/filesystem.py	2009-02-18 10:36:48 +0000
@@ -32,7 +32,7 @@
             is_root = False
         branch_app = BranchWSGIApp(
             branch, name, {'cachepath': sql_dir}, self.root.graph_cache,
-            is_root=is_root)
+            is_root=is_root, served_url='lp:loggerhead')
         return branch_app.app
 
     def app_for_non_branch(self, environ):

=== modified file 'loggerhead/controllers/__init__.py'
--- loggerhead/controllers/__init__.py	2009-02-06 05:36:03 +0000
+++ loggerhead/controllers/__init__.py	2009-02-11 03:23:45 +0000
@@ -121,7 +121,7 @@
         h = self._history
         if h is None:
             return None
-        if len(self.args) > 0:
+        if len(self.args) > 0 and self.args != ['']:
             return h.fix_revid(self.args[0])
         else:
             return h.last_revid

=== modified file 'loggerhead/controllers/inventory_ui.py'
--- loggerhead/controllers/inventory_ui.py	2008-12-05 18:52:44 +0000
+++ loggerhead/controllers/inventory_ui.py	2009-02-11 03:25:46 +0000
@@ -22,8 +22,9 @@
 import posixpath
 import urllib
 
-from paste.httpexceptions import HTTPServerError
+from paste.httpexceptions import HTTPNotFound
 
+from bzrlib import errors
 from bzrlib.revision import is_null as is_null_rev
 
 from loggerhead import util
@@ -35,8 +36,7 @@
 
 def dirname(path):
     if path is not None:
-        while path.endswith('/'):
-            path = path[:-1]
+        path = path.rstrip('/')
         path = urllib.quote(posixpath.dirname(path))
     return path
 
@@ -45,15 +45,65 @@
 
     template_path = 'loggerhead.templates.inventory'
 
+    def get_filelist(self, inv, path, sort_type='filename'):
+        """
+        return the list of all files (and their attributes) within a given
+        path subtree.
+
+        @param inv: The inventory.
+        @param path: The path of a directory within the inventory.
+        @param sort_type: How to sort the results... XXX.
+        """
+        file_id = inv.path2id(path)
+        dir_ie = inv[file_id]
+        file_list = []
+
+        revid_set = set()
+
+        for filename, entry in dir_ie.children.iteritems():
+            revid_set.add(entry.revision)
+
+        change_dict = {}
+        for change in self._history.get_changes(list(revid_set)):
+            change_dict[change.revid] = change
+
+        for filename, entry in dir_ie.children.iteritems():
+            pathname = filename
+            if entry.kind == 'directory':
+                pathname += '/'
+            if path == '':
+                absolutepath = pathname
+            else:
+                absolutepath = urllib.quote(path + '/' + pathname)
+            revid = entry.revision
+
+            file = util.Container(
+                filename=filename, executable=entry.executable,
+                kind=entry.kind, absolutepath=absolutepath,
+                file_id=entry.file_id, size=entry.text_size, revid=revid,
+                change=change_dict[revid])
+            file_list.append(file)
+
+        if sort_type == 'filename':
+            file_list.sort(key=lambda x: x.filename.lower()) # case-insensitive
+        elif sort_type == 'size':
+            file_list.sort(key=lambda x: x.size)
+        elif sort_type == 'date':
+            file_list.sort(key=lambda x: x.change.date)
+
+        # Always sort directories first.
+        file_list.sort(key=lambda x: x.kind != 'directory')
+
+        return file_list
+
     def get_values(self, path, kwargs, headers):
         history = self._history
-        revid = self.get_revid()
-
+        branch = history._branch
         try:
-            inv = history.get_inventory(revid)
-        except:
-            self.log.exception('Exception fetching changes')
-            raise HTTPServerError('Could not fetch changes')
+            revid = self.get_revid()
+            rev_tree = branch.repository.revision_tree(revid)
+        except errors.NoSuchRevision:
+            raise HTTPNotFound()
 
         file_id = kwargs.get('file_id', None)
         start_revid = kwargs.get('start_revid', None)
@@ -62,25 +112,24 @@
         # no navbar for revisions
         navigation = util.Container()
 
-        change = history.get_changes([ revid ])[0]
-        # add parent & merge-point branch-nick info, in case it's useful
-        history.get_branch_nicks([ change ])
-
         if path is not None:
-            if not path.startswith('/'):
-                path = '/' + path
-            file_id = history.get_file_id(revid, path)
+            file_id = rev_tree.path2id(path)
+            if file_id is None:
+                raise HTTPNotFound()
         else:
-            path = inv.id2path(file_id)
-
-        if file_id is None:
-            file_id = inv.root.file_id
+            if file_id is None:
+                path = ''
+            else:
+                try:
+                    path = rev_tree.id2path(file_id)
+                except errors.NoSuchId:
+                    raise HTTPNotFound()
 
         # Are we at the top of the tree
-        if inv.is_root(file_id):
+        if path in ['/', '']:
             updir = None
         else:
-            updir = dirname(path)[1:]
+            updir = dirname(path)
 
         # Directory Breadcrumbs
         directory_breadcrumbs = util.directory_breadcrumbs(
@@ -89,15 +138,10 @@
                 'files')
 
         if not is_null_rev(revid):
-            try:
-                inv = history.get_inventory(revid)
-            except:
-                self.log.exception('Exception fetching changes')
-                raise HTTPServerError('Could not fetch changes')
 
             change = history.get_changes([ revid ])[0]
             # If we're looking at the tip, use head: in the URL instead
-            if revid == history.last_revid:
+            if revid == branch.last_revision():
                 revno_url = 'head:'
             else:
                 revno_url = history.get_revno(revid)
@@ -105,16 +149,14 @@
             history.get_branch_nicks([ change ])
 
             # Create breadcrumb trail for the path within the branch
-            branch_breadcrumbs = util.branch_breadcrumbs(path, inv, 'files')
-            filelist = history.get_filelist(inv, file_id, sort_type)
+            branch_breadcrumbs = util.branch_breadcrumbs(path, rev_tree, 'files')
+            filelist = self.get_filelist(rev_tree.inventory, path, sort_type)
         else:
-            inv = None
-            file_id = None
             start_revid = None
-            sort_type = None
             change = None
             path = "/"
             updir = None
+            revno_url = 'head:'
             branch_breadcrumbs = []
             filelist = []
 
@@ -124,12 +166,9 @@
             'revid': revid,
             'revno_url': revno_url,
             'change': change,
-            'file_id': file_id,
             'path': path,
             'updir': updir,
             'filelist': filelist,
-            'history': history,
-            'posixpath': posixpath,
             'navigation': navigation,
             'url': self._branch.context_url,
             'start_revid': start_revid,

=== modified file 'loggerhead/controllers/revision_ui.py'
--- loggerhead/controllers/revision_ui.py	2008-12-05 18:52:44 +0000
+++ loggerhead/controllers/revision_ui.py	2009-02-13 02:47:13 +0000
@@ -17,19 +17,205 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
 
+import re
+from StringIO import StringIO
+
+import bzrlib.diff
+
 from paste.httpexceptions import HTTPServerError
 
 from loggerhead import util
 from loggerhead.controllers import TemplatedBranchView
+from loggerhead.history import rich_filename
 
 
 DEFAULT_LINE_COUNT_LIMIT = 3000
 
+def _process_side_by_side_buffers(line_list, delete_list, insert_list):
+    while len(delete_list) < len(insert_list):
+        delete_list.append((None, '', 'context'))
+    while len(insert_list) < len(delete_list):
+        insert_list.append((None, '', 'context'))
+    while len(delete_list) > 0:
+        d = delete_list.pop(0)
+        i = insert_list.pop(0)
+        line_list.append(util.Container(old_lineno=d[0], new_lineno=i[0],
+                                        old_line=d[1], new_line=i[1],
+                                        old_type=d[2], new_type=i[2]))
+
+
+def _make_side_by_side(chunk_list):
+    """
+    turn a normal unified-style diff (post-processed by parse_delta) into a
+    side-by-side diff structure.  the new structure is::
+
+        chunks: list(
+            diff: list(
+                old_lineno: int,
+                new_lineno: int,
+                old_line: str,
+                new_line: str,
+                type: str('context' or 'changed'),
+            )
+        )
+    """
+    out_chunk_list = []
+    for chunk in chunk_list:
+        line_list = []
+        wrap_char = '<wbr/>'
+        delete_list, insert_list = [], []
+        for line in chunk.diff:
+            # Add <wbr/> every X characters so we can wrap properly
+            wrap_line = re.findall(r'.{%d}|.+$' % 78, line.line)
+            wrap_lines = [util.html_clean(_line) for _line in wrap_line]
+            wrapped_line = wrap_char.join(wrap_lines)
+
+            if line.type == 'context':
+                if len(delete_list) or len(insert_list):
+                    _process_side_by_side_buffers(line_list, delete_list,
+                                                  insert_list)
+                    delete_list, insert_list = [], []
+                line_list.append(util.Container(old_lineno=line.old_lineno,
+                                                new_lineno=line.new_lineno,
+                                                old_line=wrapped_line,
+                                                new_line=wrapped_line,
+                                                old_type=line.type,
+                                                new_type=line.type))
+            elif line.type == 'delete':
+                delete_list.append((line.old_lineno, wrapped_line, line.type))
+            elif line.type == 'insert':
+                insert_list.append((line.new_lineno, wrapped_line, line.type))
+        if len(delete_list) or len(insert_list):
+            _process_side_by_side_buffers(line_list, delete_list, insert_list)
+        out_chunk_list.append(util.Container(diff=line_list))
+    return out_chunk_list
 
 class RevisionUI(TemplatedBranchView):
 
     template_path = 'loggerhead.templates.revision'
 
+    def _process_diff(self, diff):
+        # doesn't really need to be a method; could be static.
+        chunks = []
+        chunk = None
+        for line in diff.splitlines():
+            if len(line) == 0:
+                continue
+            if line.startswith('+++ ') or line.startswith('--- '):
+                continue
+            if line.startswith('@@ '):
+                # new chunk
+                if chunk is not None:
+                    chunks.append(chunk)
+                chunk = util.Container()
+                chunk.diff = []
+                split_lines = line.split(' ')[1:3]
+                lines = [int(x.split(',')[0][1:]) for x in split_lines]
+                old_lineno = lines[0]
+                new_lineno = lines[1]
+            elif line.startswith(' '):
+                chunk.diff.append(util.Container(old_lineno=old_lineno,
+                                                 new_lineno=new_lineno,
+                                                 type='context',
+                                                 line=line[1:]))
+                old_lineno += 1
+                new_lineno += 1
+            elif line.startswith('+'):
+                chunk.diff.append(util.Container(old_lineno=None,
+                                                 new_lineno=new_lineno,
+                                                 type='insert', line=line[1:]))
+                new_lineno += 1
+            elif line.startswith('-'):
+                chunk.diff.append(util.Container(old_lineno=old_lineno,
+                                                 new_lineno=None,
+                                                 type='delete', line=line[1:]))
+                old_lineno += 1
+            else:
+                chunk.diff.append(util.Container(old_lineno=None,
+                                                 new_lineno=None,
+                                                 type='unknown',
+                                                 line=repr(line)))
+        if chunk is not None:
+            chunks.append(chunk)
+        return chunks
+
+    def _parse_diffs(self, old_tree, new_tree, delta):
+        """
+        Return a list of processed diffs, in the format::
+
+            list(
+                filename: str,
+                file_id: str,
+                chunks: list(
+                    diff: list(
+                        old_lineno: int,
+                        new_lineno: int,
+                        type: str('context', 'delete', or 'insert'),
+                        line: str,
+                    ),
+                ),
+            )
+        """
+        process = []
+        out = []
+
+        for old_path, new_path, fid, \
+            kind, text_modified, meta_modified in delta.renamed:
+            if text_modified:
+                process.append((old_path, new_path, fid, kind))
+        for path, fid, kind, text_modified, meta_modified in delta.modified:
+            process.append((path, path, fid, kind))
+
+        for old_path, new_path, fid, kind in process:
+            old_lines = old_tree.get_file_lines(fid)
+            new_lines = new_tree.get_file_lines(fid)
+            buffer = StringIO()
+            if old_lines != new_lines:
+                try:
+                    bzrlib.diff.internal_diff(old_path, old_lines,
+                                              new_path, new_lines, buffer)
+                except bzrlib.errors.BinaryFile:
+                    diff = ''
+                else:
+                    diff = buffer.getvalue()
+            else:
+                diff = ''
+            out.append(util.Container(
+                          filename=rich_filename(new_path, kind),
+                          file_id=fid,
+                          chunks=self._process_diff(diff),
+                          raw_diff=diff))
+
+        return out
+
+    def get_change_with_diff(self, revid, compare_revid):
+        h = self._history
+        change = h.get_changes([revid])[0]
+
+        if compare_revid is None:
+            if change.parents:
+                compare_revid = change.parents[0].revid
+            else:
+                compare_revid = 'null:'
+
+        rev_tree1 = h._branch.repository.revision_tree(compare_revid)
+        rev_tree2 = h._branch.repository.revision_tree(revid)
+        delta = rev_tree2.changes_from(rev_tree1)
+
+        change.changes = h.parse_delta(delta)
+        change.changes.modified = self._parse_diffs(rev_tree1,
+                                                    rev_tree2,
+                                                    delta)
+
+        return change
+
+    @staticmethod
+    def add_side_by_side(changes):
+        # FIXME: this is a rotten API.
+        for change in changes:
+            for m in change.changes.modified:
+                m.sbs_chunks = _make_side_by_side(m.chunks)
+
     def get_values(self, path, kwargs, headers):
         h = self._history
         revid = self.get_revid()
@@ -57,7 +243,7 @@
             navigation.query = query
         util.fill_in_navigation(navigation)
 
-        change = h.get_change_with_diff(revid, compare_revid)
+        change = self.get_change_with_diff(revid, compare_revid)
         # add parent & merge-point branch-nick info, in case it's useful
         h.get_branch_nicks([change])
 
@@ -71,7 +257,7 @@
         # FIXME: not currently in use. Should be
         side_by_side = not kwargs.get('unified', False)
         if side_by_side:
-            h.add_side_by_side([change])
+            self.add_side_by_side([change])
 
         # Directory Breadcrumbs
         directory_breadcrumbs = (

=== modified file 'loggerhead/history.py'
--- loggerhead/history.py	2008-12-28 05:10:10 +0000
+++ loggerhead/history.py	2009-02-13 02:47:13 +0000
@@ -67,67 +67,6 @@
 
 bzrlib.ui.ui_factory = ThreadSafeUIFactory()
 
-
-def _process_side_by_side_buffers(line_list, delete_list, insert_list):
-    while len(delete_list) < len(insert_list):
-        delete_list.append((None, '', 'context'))
-    while len(insert_list) < len(delete_list):
-        insert_list.append((None, '', 'context'))
-    while len(delete_list) > 0:
-        d = delete_list.pop(0)
-        i = insert_list.pop(0)
-        line_list.append(util.Container(old_lineno=d[0], new_lineno=i[0],
-                                        old_line=d[1], new_line=i[1],
-                                        old_type=d[2], new_type=i[2]))
-
-
-def _make_side_by_side(chunk_list):
-    """
-    turn a normal unified-style diff (post-processed by parse_delta) into a
-    side-by-side diff structure.  the new structure is::
-
-        chunks: list(
-            diff: list(
-                old_lineno: int,
-                new_lineno: int,
-                old_line: str,
-                new_line: str,
-                type: str('context' or 'changed'),
-            )
-        )
-    """
-    out_chunk_list = []
-    for chunk in chunk_list:
-        line_list = []
-        wrap_char = '<wbr/>'
-        delete_list, insert_list = [], []
-        for line in chunk.diff:
-            # Add <wbr/> every X characters so we can wrap properly
-            wrap_line = re.findall(r'.{%d}|.+$' % 78, line.line)
-            wrap_lines = [util.html_clean(_line) for _line in wrap_line]
-            wrapped_line = wrap_char.join(wrap_lines)
-
-            if line.type == 'context':
-                if len(delete_list) or len(insert_list):
-                    _process_side_by_side_buffers(line_list, delete_list,
-                                                  insert_list)
-                    delete_list, insert_list = [], []
-                line_list.append(util.Container(old_lineno=line.old_lineno,
-                                                new_lineno=line.new_lineno,
-                                                old_line=wrapped_line,
-                                                new_line=wrapped_line,
-                                                old_type=line.type,
-                                                new_type=line.type))
-            elif line.type == 'delete':
-                delete_list.append((line.old_lineno, wrapped_line, line.type))
-            elif line.type == 'insert':
-                insert_list.append((line.new_lineno, wrapped_line, line.type))
-        if len(delete_list) or len(insert_list):
-            _process_side_by_side_buffers(line_list, delete_list, insert_list)
-        out_chunk_list.append(util.Container(diff=line_list))
-    return out_chunk_list
-
-
 def is_branch(folder):
     try:
         bzrlib.branch.Branch.open(folder)
@@ -654,26 +593,6 @@
         for entry, changes in zip(entries, changes_list):
             entry.changes = changes
 
-    def get_change_with_diff(self, revid, compare_revid=None):
-        change = self.get_changes([revid])[0]
-
-        if compare_revid is None:
-            if change.parents:
-                compare_revid = change.parents[0].revid
-            else:
-                compare_revid = 'null:'
-
-        rev_tree1 = self._branch.repository.revision_tree(compare_revid)
-        rev_tree2 = self._branch.repository.revision_tree(revid)
-        delta = rev_tree2.changes_from(rev_tree1)
-
-        change.changes = self.parse_delta(delta)
-        change.changes.modified = self._parse_diffs(rev_tree1,
-                                                    rev_tree2,
-                                                    delta)
-
-        return change
-
     def get_file(self, file_id, revid):
         "returns (path, filename, data)"
         inv = self.get_inventory(revid)
@@ -684,100 +603,6 @@
             path = '/' + path
         return path, inv_entry.name, rev_tree.get_file_text(file_id)
 
-    def _parse_diffs(self, old_tree, new_tree, delta):
-        """
-        Return a list of processed diffs, in the format::
-
-            list(
-                filename: str,
-                file_id: str,
-                chunks: list(
-                    diff: list(
-                        old_lineno: int,
-                        new_lineno: int,
-                        type: str('context', 'delete', or 'insert'),
-                        line: str,
-                    ),
-                ),
-            )
-        """
-        process = []
-        out = []
-
-        for old_path, new_path, fid, \
-            kind, text_modified, meta_modified in delta.renamed:
-            if text_modified:
-                process.append((old_path, new_path, fid, kind))
-        for path, fid, kind, text_modified, meta_modified in delta.modified:
-            process.append((path, path, fid, kind))
-
-        for old_path, new_path, fid, kind in process:
-            old_lines = old_tree.get_file_lines(fid)
-            new_lines = new_tree.get_file_lines(fid)
-            buffer = StringIO()
-            if old_lines != new_lines:
-                try:
-                    bzrlib.diff.internal_diff(old_path, old_lines,
-                                              new_path, new_lines, buffer)
-                except bzrlib.errors.BinaryFile:
-                    diff = ''
-                else:
-                    diff = buffer.getvalue()
-            else:
-                diff = ''
-            out.append(util.Container(
-                          filename=rich_filename(new_path, kind),
-                          file_id=fid,
-                          chunks=self._process_diff(diff),
-                          raw_diff=diff))
-
-        return out
-
-    def _process_diff(self, diff):
-        # doesn't really need to be a method; could be static.
-        chunks = []
-        chunk = None
-        for line in diff.splitlines():
-            if len(line) == 0:
-                continue
-            if line.startswith('+++ ') or line.startswith('--- '):
-                continue
-            if line.startswith('@@ '):
-                # new chunk
-                if chunk is not None:
-                    chunks.append(chunk)
-                chunk = util.Container()
-                chunk.diff = []
-                split_lines = line.split(' ')[1:3]
-                lines = [int(x.split(',')[0][1:]) for x in split_lines]
-                old_lineno = lines[0]
-                new_lineno = lines[1]
-            elif line.startswith(' '):
-                chunk.diff.append(util.Container(old_lineno=old_lineno,
-                                                 new_lineno=new_lineno,
-                                                 type='context',
-                                                 line=line[1:]))
-                old_lineno += 1
-                new_lineno += 1
-            elif line.startswith('+'):
-                chunk.diff.append(util.Container(old_lineno=None,
-                                                 new_lineno=new_lineno,
-                                                 type='insert', line=line[1:]))
-                new_lineno += 1
-            elif line.startswith('-'):
-                chunk.diff.append(util.Container(old_lineno=old_lineno,
-                                                 new_lineno=None,
-                                                 type='delete', line=line[1:]))
-                old_lineno += 1
-            else:
-                chunk.diff.append(util.Container(old_lineno=None,
-                                                 new_lineno=None,
-                                                 type='unknown',
-                                                 line=repr(line)))
-        if chunk is not None:
-            chunks.append(chunk)
-        return chunks
-
     def parse_delta(self, delta):
         """
         Return a nested data structure containing the changes in a delta::
@@ -816,66 +641,6 @@
         return util.Container(added=added, renamed=renamed,
                               removed=removed, modified=modified)
 
-    @staticmethod
-    def add_side_by_side(changes):
-        # FIXME: this is a rotten API.
-        for change in changes:
-            for m in change.changes.modified:
-                m.sbs_chunks = _make_side_by_side(m.chunks)
-
-    def get_filelist(self, inv, file_id, sort_type=None):
-        """
-        return the list of all files (and their attributes) within a given
-        path subtree.
-        """
-
-        dir_ie = inv[file_id]
-        path = inv.id2path(file_id)
-        file_list = []
-
-        revid_set = set()
-
-        for filename, entry in dir_ie.children.iteritems():
-            revid_set.add(entry.revision)
-
-        change_dict = {}
-        for change in self.get_changes(list(revid_set)):
-            change_dict[change.revid] = change
-
-        for filename, entry in dir_ie.children.iteritems():
-            pathname = filename
-            if entry.kind == 'directory':
-                pathname += '/'
-            if path == '':
-                absolutepath = pathname
-            else:
-                absolutepath = urllib.quote(path + '/' + pathname)
-            revid = entry.revision
-
-            file = util.Container(
-                filename=filename, executable=entry.executable,
-                kind=entry.kind, pathname=pathname, absolutepath=absolutepath,
-                file_id=entry.file_id, size=entry.text_size, revid=revid,
-                change=change_dict[revid])
-            file_list.append(file)
-
-        if sort_type == 'filename' or sort_type is None:
-            file_list.sort(key=lambda x: x.filename.lower()) # case-insensitive
-        elif sort_type == 'size':
-            file_list.sort(key=lambda x: x.size)
-        elif sort_type == 'date':
-            file_list.sort(key=lambda x: x.change.date)
-
-        # Always sort by kind to get directories first
-        file_list.sort(key=lambda x: x.kind != 'directory')
-
-        parity = 0
-        for file in file_list:
-            file.parity = parity
-            parity ^= 1
-
-        return file_list
-
     def annotate_file(self, file_id, revid):
         z = time.time()
         lineno = 1

=== modified file 'loggerhead/static/css/diff.css'
--- loggerhead/static/css/diff.css	2009-01-23 19:19:39 +0000
+++ loggerhead/static/css/diff.css	2009-02-17 02:19:38 +0000
@@ -115,6 +115,14 @@
 	background:#fff;
 	font:normal 12px/1.8 monospace;
 }
+.lineCode-u {
+	float:left;
+	width:88%;
+	padding:0px 1%; 
+	color:#000;
+	background:#fff;
+	font:normal 12px/1.8 monospace;
+}
 .delete, .insert {
 	display:block;
 	background:#ff7f7f;

=== modified file 'loggerhead/templatefunctions.py'
--- loggerhead/templatefunctions.py	2009-01-23 20:39:54 +0000
+++ loggerhead/templatefunctions.py	2009-02-18 10:36:48 +0000
@@ -54,7 +54,10 @@
 
 @templatefunc
 def branchinfo(branch):
-    return _pt('branchinfo').expand(branch=branch, **templatefunctions)
+    if branch.served_url is not None:
+        return _pt('branchinfo').expand(branch=branch, **templatefunctions)
+    else:
+        return ''
 
 
 @templatefunc

=== modified file 'loggerhead/templates/inventory.pt'
--- loggerhead/templates/inventory.pt	2009-01-23 20:53:40 +0000
+++ loggerhead/templates/inventory.pt	2009-02-10 04:25:11 +0000
@@ -64,7 +64,7 @@
 
             <!-- Show this if it's a directory -->
             <tal:block repeat="file filelist">
-            <tr tal:attributes="class string:blueRow${file/parity}" tal:condition="python:file.kind=='directory'">
+            <tr tal:attributes="class string:blueRow${repeat/file/even}" tal:condition="python:file.kind=='directory'">
                 <td class="autcell"><a tal:attributes="href python:url(['/files', revno_url, file.absolutepath])">
                         <img tal:attributes="src python:branch.static_url('/static/images/ico_folder.gif');
                                              title string:Go to ${file/filename}" />
@@ -88,7 +88,7 @@
             </tr>
 
             <!-- Show this if it's a symlink -->
-            <tr tal:attributes="class string:blueRow${file/parity}" tal:condition="python:file.kind=='symlink'">
+            <tr tal:attributes="class string:blueRow${repeat/file/even}" tal:condition="python:file.kind=='symlink'">
                 <td class="autcell"><a tal:attributes="href python:url(['/annotate', change.revno, file.absolutepath])">
                         <img tal:attributes="src python:branch.static_url('/static/images/ico_flecha.gif')" alt="Symlink" />
                     </a>
@@ -112,7 +112,7 @@
             </tr>
 
             <!-- Show this if it's a regular file -->
-            <tr tal:attributes="class string:blueRow${file/parity}" tal:condition="python:file.kind=='file'">
+            <tr tal:attributes="class string:blueRow${repeat/file/even}" tal:condition="python:file.kind=='file'">
                 <td class="autcell"><a tal:attributes="href python:url(['/annotate', revno_url, file.absolutepath])">
                         <img tal:attributes="src python:branch.static_url('/static/images/ico_file.gif');
                                              title string:Annotate ${file/filename}"

=== modified file 'loggerhead/templates/revision.pt'
--- loggerhead/templates/revision.pt	2009-01-23 20:53:40 +0000
+++ loggerhead/templates/revision.pt	2009-02-17 04:06:27 +0000
@@ -26,60 +26,60 @@
             <span metal:use-macro="breadcrumbs/directory"></span>
           </tal:no-link>
           <span>: <tal:revno content="change/revno"></tal:revno>
-        <tal:compare-to condition="python:compare_revid is not None">
-          (compared to revision <tal:block content="python:history.get_revno(compare_revid)" />)
-          </tal:compare-to>
+            <tal:compare-to condition="python:compare_revid is not None">
+              (compared to revision <tal:block content="python:history.get_revno(compare_revid)" />)
+            </tal:compare-to>
         </span></h1>
         <tal:branch-info replace="structure python:branchinfo(branch)" />
         <ul id="submenuTabs">
-            <li id="first"><a tal:attributes="href python:url(['/files', change.revno]); 
-                                               title string:browse files at revision ${change/revno}"
-                               tal:content="string:browse files at revision ${change/revno}"></a></li>
-            <li tal:condition="python:remember != change.revno">
-                <a tal:attributes="href python:url(['/revision', change.revno], remember=change.revno, compare_revid=None); 
-                                   title string:compare with another revision"
-                   tal:content="string:Compare with another revision"></a></li>       
-            <li tal:condition="python:(remember is not None) and (compare_revid is None) and (change.revno != remember)" >
-                <a tal:attributes="href python:url(['/revision', change.revno], compare_revid=history.get_revno(remember))">
-                compare with revision <tal:b content="python:history.get_revno(remember)" />
-                </a>
-            </li>
-            <li>
+          <li id="first"><a tal:attributes="href python:url(['/files', change.revno]);
+                                            title string:browse files at revision ${change/revno}"
+                            tal:content="string:browse files at revision ${change/revno}"></a></li>
+          <li tal:condition="python:remember != change.revno">
+            <a tal:attributes="href python:url(['/revision', change.revno], remember=change.revno, compare_revid=None);
+                               title string:compare with another revision"
+               tal:content="string:Compare with another revision"></a></li>
+          <li tal:condition="python:(remember is not None) and (compare_revid is None) and (change.revno != remember)" >
+            <a tal:attributes="href python:url(['/revision', change.revno], compare_revid=history.get_revno(remember))">
+              compare with revision <tal:b content="python:history.get_revno(remember)" />
+            </a>
+          </li>
+          <li>
             <a tal:condition="python:len(change.parents) > 0 and compare_revid is None"
-                   tal:attributes="href python:url(['/diff', change.revno], clear=1)">download diff</a>
+               tal:attributes="href python:url(['/diff', change.revno], clear=1)">download diff</a>
             <a tal:condition="python:compare_revid is not None"
-                   tal:attributes="href python:url(['/diff', change.revno, history.get_revno(compare_revid)], clear=1)">download diff</a>
-            </li>
-            <li id="last"><a tal:attributes="href python:url(['/changes', change.revno]); 
-                                  title string:view history from revision ${change/revno}"
-                  tal:content="string:view history from revision ${change/revno}"></a></li>
+               tal:attributes="href python:url(['/diff', change.revno, history.get_revno(compare_revid)], clear=1)">download diff</a>
+          </li>
+          <li id="last"><a tal:attributes="href python:url(['/changes', change.revno]);
+                                           title string:view history from revision ${change/revno}"
+                           tal:content="string:view history from revision ${change/revno}"></a></li>
         </ul>
 
         <tal:we-are-comparing condition="python:compare_revid is not None">
-        <ul id="submenuTabs">
-          <li id="first">
+          <ul id="submenuTabs">
+            <li id="first">
               <a tal:attributes="href python:url([ '/revision', history.get_revno(compare_revid) ], remember=change.revno, compare_revid=change.revno)">
                 reverse the comparison (<tal:b content="change/revno" /> to <tal:b content="python:history.get_revno(compare_revid)" />)
               </a>
-          </li>
-          <li id="last">
+            </li>
+            <li id="last">
               <a tal:attributes="href python:url(['/revision', change.revno], remember=None, compare_revid=None)">
                 stop comparing with revision <tal:b content="python:history.get_revno(compare_revid)" />
               </a>
-          </li>
-        </ul>
+            </li>
+          </ul>
         </tal:we-are-comparing>
-        
+
         <tal:revision-info replace="structure python:revisioninfo(url, branch, change, modified_file_link_rev)" />
         <p class="expand" id="expand_all" style="display:none;"><a tal:attributes="href string:javascript:toggle_expand_all_revisionview('open')">
             <img tal:attributes="src python:branch.static_url('/static/images/treeCollapsed.png')"
-                  alt="expand all" /> expand all</a>
+                 alt="expand all" /> expand all</a>
         </p>
         <p class="expand" id="collapse_all"><a tal:attributes="href string:javascript:toggle_expand_all_revisionview('close')">
             <img tal:attributes="src python:branch.static_url('/static/images/treeExpanded.png')"
-                  alt="collapse all" /> collapse all</a>
+                 alt="collapse all" /> collapse all</a>
         </p>
-        <!-- Table -->        
+        <!-- Table -->
         <p class="codin"><img tal:attributes="src python:branch.static_url('/static/images/newCode.gif')" alt="added" /> added</p>
         <p class="codin"><img tal:attributes="src python:branch.static_url('/static/images/deleteCode.gif')" alt="removed" /> removed</p>
         <div class="clear"><!-- --></div>
@@ -88,87 +88,91 @@
 
       <div metal:fill-slot="content">
         <div class="diff"
-             tal:condition="change/changes/modified"
-             tal:define="uniqs python:{}">
-          <!-- ! side-by-side diff -->
-          <tal:block repeat="item change/changes/modified">
-
-              <tal:u define="u python:util.uniq(uniqs, item.file_id)">
-                      <div class="diffBox"><img tal:attributes="src python:branch.static_url('/static/images/treeExpanded.png');
-                                                                title python:branch.static_url('/static/images/treeCollapsed.png');
-                                                                alt python:branch.static_url('/static/images/treeExpanded.png')"
-                                                                class="expand_diff" />
-                                           <a tal:attributes="href python:url(['/annotate', change.revno], clear=1, file_id=item.file_id);
-                                                              id string:${item/filename}-s;
-                                                              title string:Annotate ${item/filename}"
-                                              tal:content="item/filename"></a></div>
-               <div id="psuedotable" class="diffinfo">
-               <tal:block repeat="chunk item/sbs_chunks">
-
-                <tal:block condition="not:repeat/chunk/start">
-                 <div class="lineNumber separate"></div>
-                 <div class="lineCode separate"></div>
-                 <div class="lineNumber separate"></div>
-                 <div class="lineCode separate"></div>
-                 <div class="clear"><!-- --></div>
-                </tal:block>
-
-                <tal:block repeat="line chunk/diff">
-                    <div id="pseudorow">
-                    <tal:block condition="line/old_lineno">
-                    <div class="lineNumber"><tal:block replace="structure python:util.fill_div(line.old_lineno)" /></div>
-                        <div tal:attributes="class string:lineCode ${line/old_type}">
-                            <tal:block replace="structure python:util.fill_div(line.old_line)" />
-                        </div>
-                    </tal:block>
-
-                    <tal:block condition="not:line/old_lineno">
-                    <div class="lineNumber"><tal:block replace="structure python:util.fill_div(line.old_lineno)" /></div>
-                        <div tal:attributes="class string:lineCode ${line/old_type}">
-                            <tal:block replace="structure python:util.fill_div(line.old_line)" />
-                        </div>
-                    </tal:block>
-
-                    <tal:block condition="line/new_lineno">
-                    <div class="lineNumber"><tal:block replace="structure python:util.fill_div(line.new_lineno)" /></div>
-                        <div tal:attributes="class string:lineCode ${line/new_type}">
-                            <tal:block replace="structure python:util.fill_div(line.new_line)" />
-                        </div>
-                    </tal:block>
-
-                    <tal:block condition="not:line/new_lineno">
-                    <div class="lineNumber"><tal:block replace="structure python:util.fill_div(line.new_lineno)" /></div>
-                        <div tal:attributes="class string:lineCode ${line/new_type}">
-                            <tal:block replace="structure python:util.fill_div(line.new_line)" />
-                        </div>
-                    </tal:block>
-                    <div class="clear"><!-- --></div>
-                    </div>
-                   </tal:block>
-                 </tal:block>
-                 </div>
-              </tal:u>
+             tal:repeat="item change/changes/modified">
+
+          <div class="diffBox">
+            <img tal:attributes="src python:branch.static_url('/static/images/treeExpanded.png');
+                                 title python:branch.static_url('/static/images/treeCollapsed.png');
+                                 alt python:branch.static_url('/static/images/treeExpanded.png')"
+                 class="expand_diff" />
+            <a tal:attributes="href python:url(['/annotate', change.revno], clear=1, file_id=item.file_id);
+                               id string:${item/filename}-s;
+                               title string:Annotate ${item/filename}"
+               tal:content="item/filename">
+            </a>
+          </div>
+          <div class="psuedotable diffinfo"
+               tal:repeat="chunk item/sbs_chunks">
+
+            <tal:block condition="not:repeat/chunk/start">
+              <div class="lineNumber separate"></div>
+              <div class="lineCode separate"></div>
+              <div class="lineNumber separate"></div>
+              <div class="lineCode separate"></div>
+              <div class="clear"><!-- --></div>
             </tal:block>
+
+            <div class="pseudorow"
+                 tal:repeat="line chunk/diff">
+              <div class="lineNumber"
+                   tal:content="structure python:util.fill_div(line.old_lineno)"></div>
+              <div tal:attributes="class string:lineCode ${line/old_type}"
+                   tal:content="structure python:util.fill_div(line.old_line)"></div>
+
+              <div class="lineNumber"
+                   tal:content="structure python:util.fill_div(line.new_lineno)"></div>
+              <div tal:attributes="class string:lineCode ${line/new_type}"
+                   tal:content="structure python:util.fill_div(line.new_line)"></div>
+              <div class="clear"><!-- --></div>
+            </div>
+          </div>
+
+          <tal:comment condition="nothing">
+            <!-- here's how to do unified diffs -->
+            <div class="psuedotable diffinfo"
+                 tal:repeat="chunk item/chunks">
+
+              <tal:block condition="not:repeat/chunk/start">
+                <div class="lineNumber separate"></div>
+                <div class="lineCode separate"></div>
+                <div class="lineNumber separate"></div>
+                <div class="lineCode separate"></div>
+                <div class="clear"><!-- --></div>
+              </tal:block>
+
+              <div class="pseudorow"
+                   tal:repeat="line chunk/diff">
+                <div class="lineNumber"
+                     tal:content="structure python:util.fill_div(line.old_lineno)"></div>
+
+                <div class="lineNumber"
+                     tal:content="structure python:util.fill_div(line.new_lineno)"></div>
+                <div tal:attributes="class string:lineCode-u ${line/type}"
+                     tal:content="structure python:util.fill_div(util.html_clean(line.line))"></div>
+                <div class="clear"><!-- --></div>
+              </div>
+            </div>
+          </tal:comment>
         </div>
 
         <ul tal:condition="python:navigation.prev_page_revid or navigation.next_page_revid"
             id="pages">
-            <li tal:condition="navigation/prev_page_revid" 
-                class="previous">
-                <a tal:attributes="href navigation/prev_page_url">&laquo; Previous</a>
-            </li>
-            <!-- FIXME: Leaving this to eventually show page numbers. Can't show all of them,
-                        so some magic has to be done to just show the previous and next N page numbers
+          <li tal:condition="navigation/prev_page_revid"
+              class="previous">
+            <a tal:attributes="href navigation/prev_page_url">&laquo; Previous</a>
+          </li>
+          <!-- FIXME: Leaving this to eventually show page numbers. Can't show all of them,
+               so some magic has to be done to just show the previous and next N page numbers
 
-            <li class="active">1</li>
-            <tal:block tal:repeat="page_number python:range(navigation.page_count)">
-            <li><a href="#"
-                   tal:content="page_number"></a></li>
+               <li class="active">1</li>
+               <tal:block tal:repeat="page_number python:range(navigation.page_count)">
+                 <li><a href="#"
+                        tal:content="page_number"></a></li>
                </tal:block>-->
-            <li tal:condition="navigation/next_page_revid"
-                class="next">
-                <a tal:attributes="href navigation/next_page_url">Next &raquo;</a>
-            </li>
+          <li tal:condition="navigation/next_page_revid"
+              class="next">
+            <a tal:attributes="href navigation/next_page_url">Next &raquo;</a>
+          </li>
         </ul>
 
       </div>

=== added file 'loggerhead/tests/test_controllers.py'
--- loggerhead/tests/test_controllers.py	1970-01-01 00:00:00 +0000
+++ loggerhead/tests/test_controllers.py	2009-02-17 00:56:57 +0000
@@ -0,0 +1,56 @@
+from loggerhead.apps.branch import BranchWSGIApp
+from loggerhead.controllers.inventory_ui import InventoryUI
+from loggerhead.controllers.revision_ui import RevisionUI
+from loggerhead.tests.test_simple import BasicTests
+from loggerhead import util
+
+
+class TestInventoryUI(BasicTests):
+
+    def make_bzrbranch_and_inventory_ui_for_tree_shape(self, shape):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(shape)
+        tree.smart_add([])
+        tree.commit('')
+        tree.branch.lock_read()
+        self.addCleanup(tree.branch.unlock)
+        branch_app = BranchWSGIApp(tree.branch)
+        return tree.branch, InventoryUI(branch_app, branch_app.get_history)
+
+    def test_get_filelist(self):
+        bzrbranch, inv_ui = self.make_bzrbranch_and_inventory_ui_for_tree_shape(
+            ['filename'])
+        inv = bzrbranch.repository.get_inventory(bzrbranch.last_revision())
+        self.assertEqual(1, len(inv_ui.get_filelist(inv, '')))
+
+
+class TestRevisionUI(BasicTests):
+
+    def make_bzrbranch_and_revision_ui_for_tree_shapes(self, shape1, shape2):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents(shape1)
+        tree.smart_add([])
+        tree.commit('')
+        tree.smart_add([])
+        self.build_tree_contents(shape2)
+        tree.commit('')
+        tree.branch.lock_read()
+        self.addCleanup(tree.branch.unlock)
+        branch_app = BranchWSGIApp(tree.branch)
+        branch_app._environ = {
+            'wsgi.url_scheme':'',
+            'SERVER_NAME':'',
+            'SERVER_PORT':'80',
+            }
+        branch_app._url_base = ''
+        branch_app.friendly_name = ''
+        return tree.branch, RevisionUI(branch_app, branch_app.get_history)
+
+    def test_get_values(self):
+        branch, rev_ui = self.make_bzrbranch_and_revision_ui_for_tree_shapes(
+            [], [])
+        rev_ui.args = ['2']
+        util.set_context({})
+        self.assertIsInstance(
+            rev_ui.get_values('2', {}, []),
+            dict)

=== modified file 'loggerhead/tests/test_corners.py'
--- loggerhead/tests/test_corners.py	2008-06-20 02:20:05 +0000
+++ loggerhead/tests/test_corners.py	2009-02-10 01:58:31 +0000
@@ -12,11 +12,7 @@
         The commit adds a file called 'myfilename' containing the string
         'foo'.
         """
-        f = open(os.path.join(self.bzrbranch, filename), 'w')
-        try:
-            f.write("foo")
-        finally:
-            f.close()
+        self.build_tree_contents([(filename, 'foo')])
         self.tree.add(filename)
         self.tree.commit(message=commit_msg)
 
@@ -30,7 +26,7 @@
         self.addFileAndCommit('myfilename', msg)
 
         # Make a commit that changes the execute bit of 'myfilename'.
-        os.chmod(os.path.join(self.bzrbranch, 'myfilename'), 0755)
+        os.chmod('myfilename', 0755)
         newrevid = self.tree.commit(message='make something executable')
 
         # Check that it didn't break things.

=== modified file 'loggerhead/tests/test_simple.py'
--- loggerhead/tests/test_simple.py	2009-02-10 01:57:06 +0000
+++ loggerhead/tests/test_simple.py	2009-02-18 10:40:22 +0000
@@ -1,35 +1,36 @@
 import cgi
-import os
-import tempfile
-import shutil
 import logging
 
-import bzrlib.bzrdir
-import bzrlib.osutils
 from bzrlib.tests import TestCaseWithTransport
 from bzrlib.util.configobj.configobj import ConfigObj
 
 from loggerhead.apps.branch import BranchWSGIApp
 from paste.fixture import TestApp
+from paste.httpexceptions import HTTPExceptionHandler
+
 
 
 def test_config_root():
     from loggerhead.apps.config import Root
     config = ConfigObj()
-    app = TestApp(Root(config))
+    app = TestApp(HTTPExceptionHandler(Root(config)))
     res = app.get('/')
     res.mustcontain('loggerhead branches')
 
 
 class BasicTests(TestCaseWithTransport):
 
+    def setUp(self):
+        TestCaseWithTransport.setUp(self)
+        logging.basicConfig(level=logging.ERROR)
+        logging.getLogger('bzr').setLevel(logging.CRITICAL)
 
     def createBranch(self):
         self.tree = self.make_branch_and_tree('.')
 
-    def setUpLoggerhead(self):
-        app = TestApp(BranchWSGIApp(self.tree.branch, '').app)
-        return app
+    def setUpLoggerhead(self, **kw):
+        branch_app = BranchWSGIApp(self.tree.branch, '', **kw).app
+        return TestApp(HTTPExceptionHandler(branch_app))
 
 
 class TestWithSimpleTree(BasicTests):
@@ -52,6 +53,15 @@
         res = app.get('/changes')
         res.mustcontain(cgi.escape(self.msg))
 
+    def test_changes_branch_from(self):
+        app = self.setUpLoggerhead(served_url="lp:loggerhead")
+        res = app.get('/changes')
+        self.failUnless("To get this branch, use:" in res)
+        self.failUnless("lp:loggerhead" in res)
+        app = self.setUpLoggerhead(served_url=None)
+        res = app.get('/changes')
+        self.failIf("To get this branch, use:" in res)
+
     def test_changes_search(self):
         app = self.setUpLoggerhead()
         res = app.get('/changes', params={'q': 'foo'})
@@ -67,6 +77,24 @@
         app = self.setUpLoggerhead()
         res = app.get('/files')
         res.mustcontain('myfilename')
+        res = app.get('/files/')
+        res.mustcontain('myfilename')
+        res = app.get('/files/1')
+        res.mustcontain('myfilename')
+        res = app.get('/files/1/')
+        res.mustcontain('myfilename')
+        res = app.get('/files/1/?file_id=' + self.tree.path2id(''))
+        res.mustcontain('myfilename')
+
+    def test_inventory_bad_rev_404(self):
+        app = self.setUpLoggerhead()
+        res = app.get('/files/200', status=404)
+        res = app.get('/files/invalid-revid', status=404)
+
+    def test_inventory_bad_path_404(self):
+        app = self.setUpLoggerhead()
+        res = app.get('/files/1/hooha', status=404)
+        res = app.get('/files/1?file_id=dssadsada', status=404)
 
     def test_revision(self):
         app = self.setUpLoggerhead()
@@ -84,9 +112,9 @@
         app = self.setUpLoggerhead()
         res = app.get('/changes')
         res.mustcontain('No revisions!')
-    
+
     def test_inventory(self):
         app = self.setUpLoggerhead()
         res = app.get('/files')
-        res.mustcontain('myfilename')
+        res.mustcontain('No revisions!')
 

=== modified file 'serve-branches'
--- serve-branches	2009-02-04 22:41:28 +0000
+++ serve-branches	2009-02-11 03:04:17 +0000
@@ -144,8 +144,8 @@
     else:
         app = PrefixMiddleware(app, prefix=prefix)
 
+    app = HTTPExceptionHandler(app)
     app = ErrorHandlerApp(app)
-    app = HTTPExceptionHandler(app)
 
     if not options.user_port:
         port = '8080'

